"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertTypeToRelationType = convertTypeToRelationType;
exports.convertArgumentToRelationType = convertArgumentToRelationType;
exports.isTypesEqual = isTypesEqual;
exports.isTypeMissingNullable = isTypeMissingNullable;
exports.isTypeMissingArray = isTypeMissingArray;
exports.typeToString = typeToString;
const utils_1 = require("@typescript-eslint/utils");
const treeTraversal_js_1 = require("./treeTraversal.js");
function convertTypeToRelationType(arg) {
    var _a, _b;
    switch (arg.type) {
        case utils_1.AST_NODE_TYPES.TSTypeReference: {
            const name = arg.typeName.type === utils_1.AST_NODE_TYPES.Identifier ? arg.typeName.name : '';
            const param = (_b = (_a = arg.typeArguments) === null || _a === void 0 ? void 0 : _a.params) === null || _b === void 0 ? void 0 : _b[0];
            if (name === 'Promise' && param) {
                return Object.assign(Object.assign({}, convertTypeToRelationType(param)), { isLazy: true });
            }
            if (name === 'Relation' && param) {
                return Object.assign(Object.assign({}, convertTypeToRelationType(param)), { isWrapped: true });
            }
            return {
                name,
                isArray: false,
                isLazy: false,
                nullable: false,
                isWrapped: false,
            };
        }
        case utils_1.AST_NODE_TYPES.TSArrayType: {
            const item = convertTypeToRelationType(arg.elementType);
            return Object.assign(Object.assign({}, item), { isArray: true });
        }
        case utils_1.AST_NODE_TYPES.TSNullKeyword: {
            return { name: '', isArray: false, isLazy: false, nullable: true, isWrapped: false };
        }
        case utils_1.AST_NODE_TYPES.TSUnionType: {
            return arg.types.reduce((acc, currentNode) => {
                const current = convertTypeToRelationType(currentNode);
                return {
                    name: acc.name || current.name,
                    isArray: acc.isArray || current.isArray,
                    isLazy: acc.isLazy || current.isLazy,
                    isWrapped: acc.isWrapped || current.isWrapped,
                    nullable: acc.nullable || current.nullable,
                };
            }, {
                name: '',
                isArray: false,
                isLazy: false,
                nullable: false,
                isWrapped: false,
            });
        }
        default: {
            return { name: '', isArray: false, isLazy: false, nullable: false, isWrapped: false };
        }
    }
}
function convertArgumentToRelationType(relation, args) {
    var _a;
    const [otherEntityFn, ...restArguments] = args;
    const otherEntity = (0, treeTraversal_js_1.findReturnedValue)(otherEntityFn);
    if (!otherEntity) {
        return undefined;
    }
    // OneToMany, ManyToMany
    if (relation === 'OneToMany' || relation === 'ManyToMany') {
        return {
            name: otherEntity,
            isArray: true,
            isLazy: false,
            isWrapped: false,
            nullable: false,
        };
    }
    // OneToOne, ManyToOne
    const options = (0, treeTraversal_js_1.findObjectArgument)(restArguments);
    const parsedOptions = (0, treeTraversal_js_1.parseObjectLiteral)(options);
    return {
        name: otherEntity,
        isArray: false,
        isLazy: false,
        isWrapped: false,
        nullable: (_a = parsedOptions === null || parsedOptions === void 0 ? void 0 : parsedOptions.nullable) !== null && _a !== void 0 ? _a : true,
    };
}
function isTypesEqual(toType, tsType) {
    // Just check field equality
    return (toType.name === tsType.name &&
        toType.nullable === tsType.nullable &&
        toType.isArray === tsType.isArray);
}
// Relations are nullable by default which can be confusing, help with a custom message
function isTypeMissingNullable(toType, tsType) {
    // Check if the relation should be nullable but not
    return toType.name === tsType.name && toType.nullable && !tsType.nullable;
}
// Relations are nullable by default which can be confusing, help with a custom message
function isTypeMissingArray(toType, tsType) {
    // Check if the relation should be an array but not
    return toType.name === tsType.name && toType.isArray && !tsType.isArray;
}
function typeToString(relation, { isLazy, isWrapped }) {
    let result = relation.name;
    if (relation.isArray) {
        result += '[]';
    }
    if (relation.nullable) {
        result += ' | null';
    }
    if (isWrapped) {
        result = `Relation<${result}>`;
    }
    if (isLazy) {
        result = `Promise<${result}>`;
    }
    return result;
}
