"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findDecoratorArguments = findDecoratorArguments;
exports.findEitherDecoratorArguments = findEitherDecoratorArguments;
exports.findParentClass = findParentClass;
exports.findReturnedValue = findReturnedValue;
exports.findObjectArgument = findObjectArgument;
exports.parseObjectLiteral = parseObjectLiteral;
const utils_1 = require("@typescript-eslint/utils");
function findDecoratorArguments(decorators, name) {
    return decorators === null || decorators === void 0 ? void 0 : decorators.reduce((previous, decorator) => {
        if (decorator.expression.type === utils_1.AST_NODE_TYPES.CallExpression &&
            decorator.expression.callee.type === utils_1.AST_NODE_TYPES.Identifier &&
            decorator.expression.callee.name === name) {
            return decorator.expression.arguments;
        }
        return previous;
    }, undefined);
}
function findEitherDecoratorArguments(decorators, relationTypes) {
    for (const relation of relationTypes) {
        const relationArguments = findDecoratorArguments(decorators, relation);
        if (relationArguments) {
            return [relation, relationArguments];
        }
    }
    return undefined;
}
function findParentClass(node) {
    let parentClass = node.parent;
    while (parentClass) {
        if (parentClass.type === utils_1.AST_NODE_TYPES.ClassDeclaration) {
            return parentClass;
        }
        parentClass = parentClass.parent;
    }
    return undefined;
}
function findReturnedValue(node) {
    if ((node === null || node === void 0 ? void 0 : node.type) === utils_1.AST_NODE_TYPES.ArrowFunctionExpression &&
        node.body.type === utils_1.AST_NODE_TYPES.Identifier) {
        return node.body.name;
    }
    return undefined;
}
function findObjectArgument(args = []) {
    return args.find((arg) => arg.type === utils_1.AST_NODE_TYPES.ObjectExpression);
}
function parseObjectLiteral(objectLiteral) {
    if ((objectLiteral === null || objectLiteral === void 0 ? void 0 : objectLiteral.type) === utils_1.AST_NODE_TYPES.ObjectExpression) {
        return objectLiteral.properties.reduce((parsedObject, prop) => {
            if (prop.type === utils_1.AST_NODE_TYPES.Property &&
                prop.key.type === utils_1.AST_NODE_TYPES.Identifier) {
                if (prop.value.type === utils_1.AST_NODE_TYPES.Literal) {
                    return Object.assign(Object.assign({}, parsedObject), { [prop.key.name]: prop.value.value });
                }
                // Handle unknown values as object for transformer
                return Object.assign(Object.assign({}, parsedObject), { [prop.key.name]: {} });
            }
            return parsedObject;
        }, {});
    }
    return {};
}
